@@ -2,29 +2,46 @@ use super::Span;
 use crate::sass::{FormalArgs, Name};
 use std::fmt::{self, Write};
 use std::str::from_utf8;
+use std::sync::Arc;
 
 /// A position in sass input.
 #[derive(Clone, Debug, Eq, PartialEq, PartialOrd, Ord)]
 pub struct SourcePos {
-    /// The text of the source line containing this pos.
-    pub line: String,
+    p: Arc<SourcePosImpl>,
+}
+
+#[derive(Clone, Debug, Eq, PartialEq, PartialOrd, Ord)]
+struct SourcePosImpl {
     /// The line number of this pos.
-    pub line_no: u32,
+    line_no: u32,
     /// The position on the line.
-    pub line_pos: usize,
+    line_pos: usize,
     /// The length of the interesting part.
     length: usize,
+    /// The text of the source line containing this pos.
+    line: String,
     /// The source file name and from where it was loaded.
-    pub file: SourceName,
+    file: SourceName,
 }
 
 impl SourcePos {
     /// Create a new SourcePos from a start and an end Span.
     pub fn from_to(start: Span, end: Span) -> Self {
-        let mut result = Self::from(start);
+        let mut result = SourcePosImpl::from(start);
         result.length =
             std::cmp::max(1, end.location_offset() - start.location_offset());
-        result
+        SourcePos {
+            p: Arc::new(result),
+        }
+    }
+
+    pub(crate) fn in_call(&self, name: &str) -> Self {
+        SourcePos {
+            p: Arc::new(SourcePosImpl {
+                file: SourceName::called(name, self.clone()),
+                ..(*self.p).clone()
+            }),
+        }
     }
 
     pub(crate) fn mock_function(
@@ -34,11 +51,13 @@ impl SourcePos {
     ) -> Self {
         let args = args.to_string();
         SourcePos {
-            line: format!("@function {}{} {{", name, args),
-            line_no: 1,
-            line_pos: 11,
-            length: name.as_ref().chars().count() + args.chars().count(),
-            file: SourceName::root(module),
+            p: Arc::new(SourcePosImpl {
+                line: format!("@function {}{} {{", name, args),
+                line_no: 1,
+                line_pos: 11,
+                length: name.as_ref().chars().count() + args.chars().count(),
+                file: SourceName::root(module),
+            }),
         }
     }
 
@@ -66,10 +85,10 @@ impl SourcePos {
         marker: char,
         what: &str,
     ) -> fmt::Result {
-        let filename = if self.file.name.is_empty() {
+        let filename = if self.p.file.name.is_empty() {
             String::new()
         } else {
-            format!("--> {}", self.file.name)
+            format!("--> {}", self.p.file.name)
         };
         self.show_impl(out, &filename, marker, what)
     }
@@ -80,20 +99,28 @@ impl SourcePos {
         marker: char,
         what: &str,
     ) -> fmt::Result {
-        let line_no = self.line_no.to_string();
-        write!(
+        let lnw = self.p.line_no.to_string().len();
+        write!(out, "{0:lnw$} ,{arrow}", "", arrow = arrow, lnw = lnw)?;
+        self.show_inner(out, lnw, marker, what)?;
+        write!(out, "{0:lnw$} '", "", lnw = lnw)
+    }
+    pub(crate) fn show_inner(
+        &self,
+        out: &mut impl Write,
+        lnw: usize,
+        marker: char,
+        what: &str,
+    ) -> fmt::Result {
+        writeln!(
             out,
-            "{0:lnw$} ,{arrow}\
-             \n{ln} | {line}\
-             \n{0:lnw$} |{0:>lpos$}{mark}{what}\
-             \n{0:lnw$} '",
+            "\n{ln:<lnw$} | {line}\
+             \n{0:lnw$} |{0:>lpos$}{mark}{what}",
             "",
-            arrow = arrow,
-            line = self.line,
-            ln = line_no,
-            lnw = line_no.len(),
-            lpos = self.line_pos,
-            mark = marker.to_string().repeat(self.length),
+            line = self.p.line,
+            ln = self.p.line_no,
+            lnw = lnw,
+            lpos = self.p.line_pos,
+            mark = marker.to_string().repeat(self.p.length),
             what = what,
         )
     }
@@ -105,34 +132,50 @@ impl SourcePos {
                 out,
                 "\n{0:lnw$} {file} {row}:{col}  {cause}",
                 "",
-                lnw = self.line_no.to_string().len(),
-                file = pos.file.name(),
-                row = pos.line_no,
-                col = pos.line_pos,
-                cause = if pos.file.imported_from().is_some() {
-                    "import"
-                } else {
-                    "root stylesheet"
-                },
+                lnw = self.p.line_no.to_string().len(),
+                file = pos.p.file.name(),
+                row = pos.p.line_no,
+                col = pos.p.line_pos,
+                cause = pos.p.file.imported,
             )?;
-            nextpos = pos.file.imported_from();
+            nextpos = match &pos.p.file.imported {
+                SourceKind::Root => None,
+                SourceKind::Imported(pos) => Some(pos),
+                SourceKind::Called(_, pos) => Some(pos),
+            };
         }
         Ok(())
     }
 
     /// If self is preceded (on same line) by `s`, include `s` in self.
     pub(crate) fn opt_back(&mut self, s: &str) {
-        if self.line[..self.line_pos - 1].ends_with(s) {
+        let p: &mut SourcePosImpl = Arc::make_mut(&mut self.p);
+        if p.line[..p.line_pos - 1].ends_with(s) {
             let len = s.chars().count();
-            self.line_pos -= len;
-            self.length += len;
+            p.line_pos -= len;
+            p.length += len;
         }
     }
+
+    /// True if this is the position of something built-in.
+    pub fn is_builtin(&self) -> bool {
+        self.p.file.is_builtin()
+    }
+    pub(crate) fn same_file_as(&self, other: &Self) -> bool {
+        self.p.file.name == other.p.file.name
+    }
 }
 
 impl From<Span<'_>> for SourcePos {
     fn from(span: Span) -> Self {
         SourcePos {
+            p: Arc::new(SourcePosImpl::from(span)),
+        }
+    }
+}
+impl From<Span<'_>> for SourcePosImpl {
+    fn from(span: Span) -> Self {
+        SourcePosImpl {
             line: from_utf8(span.get_line_beginning())
                 .unwrap_or("<<failed to display line>>")
                 .trim_end()
@@ -152,7 +195,7 @@ impl From<Span<'_>> for SourcePos {
 #[derive(Clone, Debug, Eq, PartialEq, PartialOrd, Ord)]
 pub struct SourceName {
     name: String,
-    imported: Option<Box<SourcePos>>,
+    imported: SourceKind,
 }
 
 impl SourceName {
@@ -160,14 +203,21 @@ impl SourceName {
     pub fn root<T: ToString>(name: T) -> Self {
         SourceName {
             name: name.to_string(),
-            imported: None,
+            imported: SourceKind::Root,
         }
     }
     /// Create a name for a file imported from a specific pos.
     pub fn imported<T: ToString>(name: T, from: SourcePos) -> Self {
         SourceName {
             name: name.to_string(),
-            imported: Some(Box::new(from)),
+            imported: SourceKind::Imported(from),
+        }
+    }
+    /// Create a name for a mixin called from a specific pos.
+    pub fn called<T: ToString>(name: T, from: SourcePos) -> Self {
+        SourceName {
+            name: from.p.file.name.clone(),
+            imported: SourceKind::Called(name.to_string(), from),
         }
     }
 
@@ -175,8 +225,27 @@ impl SourceName {
     pub fn name(&self) -> &str {
         &self.name
     }
-    /// Get where this source is imported from, if not top-level.
-    pub fn imported_from(&self) -> Option<&SourcePos> {
-        self.imported.as_ref().map(|b| b.as_ref())
+
+    /// True if this is the position of something built-in.
+    pub fn is_builtin(&self) -> bool {
+        // Note: maybe implement this as a sepate source kind?
+        self.name.starts_with("sass:") || self.name.is_empty()
+    }
+}
+
+#[derive(Clone, Debug, Eq, PartialEq, PartialOrd, Ord)]
+enum SourceKind {
+    Root,
+    Imported(SourcePos),
+    Called(String, SourcePos),
+}
+
+impl fmt::Display for SourceKind {
+    fn fmt(&self, out: &mut fmt::Formatter) -> fmt::Result {
+        match self {
+            SourceKind::Root => out.write_str("root stylesheet"),
+            SourceKind::Imported(_) => out.write_str("import"),
+            SourceKind::Called(name, _) => write!(out, "{}()", name),
+        }
     }
 }
