@@ -1,8 +1,10 @@
 use super::{
-    check, get_checked, get_opt_check, get_string, Error, FunctionMap,
+    check, get_checked, get_opt_check, get_string, is_not, looks_like_call,
+    CheckedArg, Error, FunctionMap,
 };
-use crate::css::{CallArgs, Value};
+use crate::css::{CallArgs, CssString, Value};
 use crate::sass::{Function, Mixin, Name};
+use crate::value::Quotes;
 use crate::{Format, Scope, ScopeRef};
 
 pub fn create_module() -> Scope {
@@ -11,24 +13,45 @@ pub fn create_module() -> Scope {
     // TODO: load_css
 
     // - - - Functions - - -
+    def!(f, calc_args(calc), |s| {
+        get_checked(s, name!(calc), |v| match v {
+            Value::Call(_, args) => Ok(args.into()),
+            Value::Literal(s) if looks_like_call(&s) => {
+                let s = s.value();
+                let i = s.find('(').unwrap();
+                Ok(s[i + 1..s.len() - 1].into())
+            }
+            v => Err(is_not(&v, "a calculation")),
+        })
+    });
+    def!(f, calc_name(calc), |s| {
+        let name = get_checked(s, name!(calc), |v| match v {
+            Value::Call(name, _) => Ok(name),
+            Value::Literal(s) if looks_like_call(&s) => {
+                let s = s.value();
+                let i = s.find('(').unwrap();
+                Ok(s[..i].to_string())
+            }
+            v => Err(is_not(&v, "a calculation")),
+        })?;
+        Ok(CssString::new(name, Quotes::Double).into())
+    });
     def_va!(f, call(function, args), |s| {
         let (function, name) = match s.get("function")? {
             Value::Function(ref name, ref func) => {
                 (func.clone(), name.clone())
             }
-            Value::Literal(name, _) => {
+            Value::Literal(name) => {
                 dep_warn!(
                     "Passing a string to call() is deprecated and \
                      will be illegal"
                 );
-                (get_function(s, None, &name)?, name.clone())
+                let name = name.value();
+                (get_function(s, None, name)?, name.into())
             }
             ref v => {
-                return Err(Error::bad_arg(
-                    name!(function),
-                    v,
-                    "is not a function reference",
-                ))
+                return Err(is_not(v, "a function reference"))
+                    .named(name!(function));
             }
         };
         let args = CallArgs::from_value(s.get("args")?)?;
@@ -44,50 +67,46 @@ pub fn create_module() -> Scope {
             Ok((!Mixin::is_no_body(&content.body)).into())
         } else {
             Err(Error::error(
-                "content-exists() may only be called within a mixin",
+                "content-exists() may only be called within a mixin.",
             ))
         }
     });
     def!(f, feature_exists(feature), |s| {
-        let (feature, _q) = get_string(s, "feature")?;
-        Ok(IMPLEMENTED_FEATURES.iter().any(|s| s == &feature).into())
+        let feature = get_string(s, "feature")?;
+        Ok(IMPLEMENTED_FEATURES
+            .iter()
+            .any(|s| *s == feature.value())
+            .into())
     });
     def!(f, function_exists(name, module = b"null"), |s| {
-        let (name, _q) = get_string(s, "name")?;
-        let module =
-            get_opt_check(s, name!(module), check::string)?.map(|(s, _q)| s);
-        Ok(get_function(s, module, &name)?.is_some().into())
+        let name = get_string(s, "name")?;
+        let module = get_opt_check(s, name!(module), check::string)?;
+        Ok(get_function(s, module, name.value())?.is_some().into())
     });
     def!(
         f,
         get_function(name, css = b"false", module = b"null"),
         |s| {
-            let (v, q) = get_string(s, "name")?;
+            let name = get_string(s, "name")?;
             let module = get_opt_check(s, name!(module), check::string)?;
             if s.get("css")?.is_true() {
                 if module.is_some() {
                     return Err(Error::error(
-                        "$css and $module may not both be passed at once",
+                        "$css and $module may not both be passed at once.",
                     ));
                 }
-                Ok(Value::Function(v, None))
-            } else if let Some(f) =
-                get_function(s, module.map(|(s, _)| s), &v)?
-            {
-                Ok(Value::Function(v, Some(f)))
+                Ok(Value::Function(name.value().into(), None))
+            } else if let Some(f) = get_function(s, module, name.value())? {
+                Ok(Value::Function(name.value().into(), Some(f)))
             } else {
-                Err(Error::S(format!(
-                    "Error: Function not found: {}",
-                    Value::Literal(v, q).format(Format::introspect())
-                )))
+                Err(Error::S(format!("Error: Function not found: {}", name)))
             }
         }
     );
     def!(f, global_variable_exists(name, module = b"null"), |s| {
-        let (v, _q) = get_string(s, "name")?;
-        let module = get_opt_check(s, name!(module), check::string)?;
-        let module = get_scope(s, module.map(|(s, _q)| s), true)?;
-        Ok(module.get_global_or_none(&v.into()).is_some().into())
+        let name = get_string(s, "name")?;
+        let module = get_module_arg(s, true)?;
+        Ok(module.get_global_or_none(&name.into()).is_some().into())
     });
     def!(f, inspect(value), |s| {
         Ok(s.get("value")?
@@ -98,10 +117,7 @@ pub fn create_module() -> Scope {
     def!(f, keywords(args), |s| {
         let args = get_checked(s, name!(args), |v| match v {
             Value::ArgList(args) => Ok(args.named),
-            v => Err(format!(
-                "{} is not an argument list",
-                v.format(Format::introspect())
-            )),
+            v => Err(is_not(&v, "an argument list")),
         })?;
         Ok(Value::Map(
             args.into_iter()
@@ -110,27 +126,24 @@ pub fn create_module() -> Scope {
         ))
     });
     def!(f, mixin_exists(name, module = b"null"), |s| {
-        let (v, _q) = get_string(s, "name")?;
-        let module = get_opt_check(s, name!(module), check::string)?;
-        let module = get_scope(s, module.map(|(s, _q)| s), true)?;
-        Ok(module.get_mixin(&v.into()).is_some().into())
+        let name = get_string(s, "name")?.into();
+        let module = get_module_arg(s, true)?;
+        Ok(module.get_mixin(&name).is_some().into())
     });
     def!(f, module_functions(module), |s| {
-        let module = get_opt_check(s, name!(module), check::string)?;
-        let module = get_scope(s, module.map(|(s, _q)| s), false)?;
+        let module = get_module_arg(s, false)?;
         Ok(module.functions_map())
     });
     def!(f, module_variables(module), |s| {
-        let module = get_opt_check(s, name!(module), check::string)?;
-        let module = get_scope(s, module.map(|(s, _q)| s), false)?;
+        let module = get_module_arg(s, false)?;
         Ok(module.variables_map())
     });
     def!(f, type_of(value), |s| {
         Ok(s.get("value")?.type_name().into())
     });
     def!(f, variable_exists(name), |s| {
-        let (v, _q) = get_string(s, "name")?;
-        Ok(call_scope(s).get_or_none(&v.into()).is_some().into())
+        let name = get_string(s, "name")?.into();
+        Ok(call_scope(s).get_or_none(&name).is_some().into())
     });
     f
 }
@@ -149,7 +162,7 @@ pub fn expose(m: &Scope, global: &mut FunctionMap) {
         (name!(type_of), name!(type_of)),
         (name!(variable_exists), name!(variable_exists)),
     ] {
-        global.insert(gname.clone(), m.get_lfunction(&lname));
+        global.insert(gname.clone(), m.get_lfunction(lname));
     }
 }
 
@@ -169,6 +182,11 @@ static IMPLEMENTED_FEATURES: &[&str] = &[
     // "custom-property",
 ];
 
+fn get_module_arg(s: &Scope, use_the: bool) -> Result<ScopeRef, Error> {
+    let module = get_opt_check(s, name!(module), check::string)?;
+    get_scope(s, module, use_the)
+}
+
 fn call_scope(s: &Scope) -> ScopeRef {
     s.get_module("%%CALLING_SCOPE%%").unwrap()
 }
@@ -176,19 +194,17 @@ fn call_scope(s: &Scope) -> ScopeRef {
 // Note: `the` is compensating for an inconsistensy in sass-spec
 fn get_scope(
     s: &Scope,
-    module: Option<String>,
+    module: Option<CssString>,
     the: bool,
 ) -> Result<ScopeRef, Error> {
     if let Some(module) = module {
-        if let Some(module) = call_scope(s).get_module(&module) {
-            Ok(module)
-        } else {
-            Err(Error::error(format!(
-                "There is no module with {}namespace {:?}",
+        call_scope(s).get_module(module.value()).ok_or_else(|| {
+            Error::error(format!(
+                "There is no module with {}namespace {}.",
                 if the { "the " } else { "" },
                 module
-            )))
-        }
+            ))
+        })
     } else {
         Ok(call_scope(s))
     }
@@ -196,12 +212,11 @@ fn get_scope(
 
 fn get_function(
     s: &Scope,
-    module: Option<String>,
+    module: Option<CssString>,
     name: &str,
 ) -> Result<Option<Function>, Error> {
     if let Some(module) = module {
-        let module = get_scope(s, Some(module), true)?;
-        module.get_function(&name.into())
+        get_scope(s, Some(module), true)?.get_function(&name.into())
     } else {
         let name = name.into();
         Ok(call_scope(s)
