@@ -1,9 +1,9 @@
-use crate::css::{CallArgs, Value};
+use crate::css::{is_not, CallArgs, CssString, Value};
 use crate::error::Error;
 use crate::output::{Format, Formatted};
 use crate::parser::SourcePos;
 use crate::sass::{FormalArgs, Name};
-use crate::value::{ListSeparator, Numeric, Quotes};
+use crate::value::Numeric;
 use crate::{sass, Scope, ScopeRef};
 use lazy_static::lazy_static;
 use std::collections::BTreeMap;
@@ -282,22 +282,12 @@ fn get_integer(s: &Scope, name: Name) -> Result<i64, Error> {
     get_checked(s, name, check::unitless_int)
 }
 
-fn get_string(
-    s: &Scope,
-    name: &'static str,
-) -> Result<(String, Quotes), Error> {
+fn get_string(s: &Scope, name: &'static str) -> Result<CssString, Error> {
     get_checked(s, name.into(), check::string)
 }
 
 fn get_va_list(s: &Scope, name: Name) -> Result<Vec<Value>, Error> {
-    match s.get(name.as_ref())? {
-        Value::ArgList(args) => {
-            args.check_no_named()?;
-            Ok(args.positional)
-        }
-        Value::List(v, Some(ListSeparator::Comma), _) => Ok(v),
-        single => Ok(vec![single]),
-    }
+    get_checked(s, name, check::va_list)
 }
 
 fn expected_to<'a, T>(value: &'a T, cond: &str) -> String
@@ -305,7 +295,7 @@ where
     Formatted<'a, T>: std::fmt::Display,
 {
     format!(
-        "Expected {} to {}",
+        "Expected {} to {}.",
         Formatted {
             value,
             format: Format::introspect()
@@ -314,21 +304,34 @@ where
     )
 }
 
+fn is_special(v: &Value) -> bool {
+    match v {
+        Value::Call(..) => true,
+        Value::Literal(s) if looks_like_call(s) => true,
+        Value::BinOp(..) => true,
+        _ => false,
+    }
+}
+
+fn looks_like_call(s: &CssString) -> bool {
+    s.quotes().is_none()
+        && s.value().contains('(')
+        && s.value().ends_with(')')
+}
+
 mod check {
-    use super::expected_to;
-    use crate::css::Value;
-    use crate::output::Format;
-    use crate::value::{Number, Numeric, Quotes};
+    use super::{expected_to, is_not};
+    use crate::css::{CssString, Value};
+    use crate::value::{ListSeparator, Number, Numeric};
 
     pub fn numeric(v: Value) -> Result<Numeric, String> {
-        v.numeric_value().map_err(|v| {
-            format!("{} is not a number", v.format(Format::introspect()))
-        })
+        v.numeric_value().map_err(|v| is_not(&v, "a number"))
     }
     pub fn int(v: Value) -> Result<i64, String> {
-        numeric(v)?.value.into_integer().map_err(|v| {
-            format!("{} is not an int", v.format(Format::introspect()))
-        })
+        numeric(v)?
+            .value
+            .into_integer()
+            .map_err(|v| is_not(&v, "an int"))
     }
 
     pub fn unitless(v: Value) -> Result<Number, String> {
@@ -340,19 +343,38 @@ mod check {
         }
     }
     pub fn unitless_int(v: Value) -> Result<i64, String> {
-        let v0 = unitless(v)?;
-        v0.into_integer().map_err(|v| {
-            format!("{} is not an int", v.format(Format::introspect()))
-        })
+        unitless(v)?
+            .into_integer()
+            .map_err(|v| is_not(&v, "an int"))
     }
 
-    pub fn string(v: Value) -> Result<(String, Quotes), String> {
+    pub fn string(v: Value) -> Result<CssString, String> {
         match v {
-            Value::Literal(s, q) => Ok((s, q)),
-            v => Err(format!(
-                "{} is not a string",
-                v.format(Format::introspect())
-            )),
+            Value::Literal(s) => Ok(s),
+            Value::Call(name, args) => {
+                Ok(format!("{}({})", name, args).into())
+            }
+            v => Err(is_not(&v, "a string")),
+        }
+    }
+
+    pub fn va_list(v: Value) -> Result<Vec<Value>, String> {
+        match v {
+            Value::ArgList(args) => {
+                args.check_no_named().map_err(|e| e.to_string())?;
+                Ok(args.positional)
+            }
+            Value::List(v, Some(ListSeparator::Comma), _) => Ok(v),
+            single => Ok(vec![single]),
+        }
+    }
+    pub fn va_list_nonempty(v: Value) -> Result<Vec<Value>, String> {
+        let result = va_list(v)?;
+        if result.is_empty() {
+            // TODO: Parameterize "selector"?  Or rename fn va_selectors?
+            Err("At least one selector must be passed.".into())
+        } else {
+            Ok(result)
         }
     }
 }
