@@ -3,7 +3,7 @@ use crate::error::Error;
 use crate::output::Format;
 use crate::parser::SourcePos;
 use crate::sass::{CallArgs, Function, SassString};
-use crate::value::{ListSeparator, Number, Numeric, Operator, Quotes, Rgba};
+use crate::value::{ListSeparator, Number, Numeric, Operator, Rgba};
 use crate::ScopeRef;
 use num_traits::Zero;
 
@@ -95,14 +95,7 @@ impl Value {
     ) -> Result<css::Value, Error> {
         match *self {
             Value::Bang(ref s) => Ok(css::Value::Bang(s.clone())),
-            Value::Literal(ref s) => {
-                let (s, q) = s.evaluate(scope)?;
-                if s.is_empty() && q == Quotes::None {
-                    Ok(css::Value::Null)
-                } else {
-                    Ok(css::Value::Literal(s, q))
-                }
-            }
+            Value::Literal(ref s) => Ok(s.evaluate(scope)?.into()),
             Value::Paren(ref v, ref expl) => {
                 let v = v.do_evaluate(scope, !expl)?;
                 if !expl && v != css::Value::Null {
@@ -139,13 +132,22 @@ impl Value {
                     return args.evaluate_single(scope, name, num);
                 }
                 let args = args.evaluate(scope.clone())?;
-                let call_err = |e: Error| match e {
-                    Error::BadArguments(msg, decl) => {
-                        Error::BadCall(msg, pos.clone(), Some(decl))
-                    }
-                    e => Error::BadCall(e.to_string(), pos.clone(), None),
-                };
                 if let Some(name) = name.single_raw() {
+                    let call_err = |e: Error| match e {
+                        Error::BadArguments(msg, decl) => {
+                            let pos = if decl.is_builtin() {
+                                pos.clone()
+                            } else {
+                                pos.in_call(name)
+                            };
+                            Error::BadCall(msg, pos, Some(decl))
+                        }
+                        Error::AtError(msg, _pos) => {
+                            let msg = format!("Error: {}", msg);
+                            Error::BadCall(msg, pos.clone(), None)
+                        }
+                        e => Error::BadCall(e.to_string(), pos.clone(), None),
+                    };
                     let name = name.into();
                     if let Some(f) = scope
                         .get_function(&name)
@@ -155,8 +157,8 @@ impl Value {
                         return f.call(scope.clone(), args).map_err(call_err);
                     }
                 }
-                let (name, _) = name.evaluate(scope)?;
-                Ok(css::Value::Call(name, args))
+                let name = name.evaluate(scope)?;
+                Ok(css::Value::Call(name.value().into(), args))
             }
             Value::Numeric(ref num) => {
                 Ok(css::Value::Numeric(num.clone(), arithmetic))
@@ -167,7 +169,7 @@ impl Value {
                     let k = k.do_evaluate(scope.clone(), arithmetic)?;
                     let v = v.do_evaluate(scope.clone(), arithmetic)?;
                     if items.insert(k, v).is_some() {
-                        return Err(Error::error("Duplicate key"));
+                        return Err(Error::error("Duplicate key."));
                     }
                 }
                 Ok(css::Value::Map(items))
@@ -175,7 +177,7 @@ impl Value {
             Value::Null => Ok(css::Value::Null),
             Value::True => Ok(css::Value::True),
             Value::False => Ok(css::Value::False),
-            Value::BinOp(ref a, _, ref op, _, ref b) => {
+            Value::BinOp(ref a, s1, ref op, s2, ref b) => {
                 if *op == Operator::And {
                     let a = a.do_evaluate(scope.clone(), true)?;
                     if a.is_true() {
@@ -210,9 +212,11 @@ impl Value {
                     Ok(op.eval(a.clone(), b.clone()).unwrap_or_else(|| {
                         css::Value::BinOp(
                             Box::new(a),
-                            false,
+                            s1 && op != &Operator::Div
+                                && op != &Operator::Minus,
                             op.clone(),
-                            false,
+                            s2 && op != &Operator::Div
+                                && op != &Operator::Minus,
                             Box::new(b),
                         )
                     }))
@@ -236,16 +240,13 @@ impl Value {
                     (Operator::Plus, css::Value::Numeric(v, _)) => {
                         Ok(css::Value::Numeric(v, true))
                     }
-                    (op, css::Value::Literal(s, Quotes::None)) => {
-                        Ok(css::Value::Literal(
-                            format!("{}{}", op, s),
-                            Quotes::None,
-                        ))
+                    (op, css::Value::Literal(s)) if s.quotes().is_none() => {
+                        Ok(format!("{}{}", op, s).into())
                     }
                     (op, v) => Ok(css::Value::UnaryOp(op, Box::new(v))),
                 }
             }
-            Value::HereSelector => Ok(scope.get_selectors().to_value()),
+            Value::HereSelector => Ok(scope.get_selectors().clone().into()),
             Value::UnicodeRange(ref s) => {
                 Ok(css::Value::UnicodeRange(s.clone()))
             }
