@@ -1,9 +1,7 @@
 //! A scope is something that contains variable values.
-
-use crate::css::Value;
+use crate::css::{CssString, Selectors, Value};
 use crate::output::Format;
 use crate::sass::{Expose, Function, Item, Mixin, Name, UseAs};
-use crate::selectors::Selectors;
 use crate::Error;
 use lazy_static::lazy_static;
 use std::collections::BTreeMap;
@@ -131,13 +129,14 @@ impl ScopeRef {
                     );
                     None
                 }
-                Item::Error(ref value) => {
-                    return Err(Error::S(format!(
-                        "Error: {}",
+                Item::Error(ref value, ref pos) => {
+                    return Err(Error::AtError(
                         value
-                            .evaluate(self.clone())?
-                            .format(self.get_format()),
-                    )));
+                            .evaluate(self)?
+                            .format(Format::introspect())
+                            .to_string(),
+                        pos.clone(),
+                    ));
                 }
                 Item::None => None,
                 Item::Comment(..) => None,
@@ -251,7 +250,7 @@ impl<'a> Scope {
     }
     pub(crate) fn get_name(&self) -> String {
         match self.get_or_none(&Name::from_static("@scope_name@")) {
-            Some(Value::Literal(s, _q)) => s,
+            Some(Value::Literal(s)) => s.value().into(),
             _ => "".into(),
         }
     }
@@ -346,7 +345,7 @@ impl<'a> Scope {
     /// Special case: in names is a single name, value is used directly.
     pub fn define_multi(&self, names: &[Name], value: &Value) {
         if names.len() == 1 {
-            self.define(names[0].clone(), &value);
+            self.define(names[0].clone(), value);
         } else {
             let values = value.clone().iter_items();
             if values.len() > names.len() {
@@ -360,7 +359,7 @@ impl<'a> Scope {
                 for name in names {
                     self.define(
                         name.clone(),
-                        &values.next().unwrap_or(&Value::Null),
+                        values.next().unwrap_or(&Value::Null),
                     )
                 }
             }
@@ -453,11 +452,11 @@ impl<'a> Scope {
                 if let Some(f) = module.get_function(&name)? {
                     Ok(Some(f))
                 } else {
-                    Err(Error::error("Undefined function"))
+                    Err(Error::error("Undefined function."))
                 }
             } else {
                 return Err(Error::error(format!(
-                    "There is no module with the namespace {:?}",
+                    "There is no module with the namespace {:?}.",
                     modulename
                 )));
             }
@@ -504,7 +503,7 @@ impl<'a> Scope {
                 let name = name
                     .rfind(|c| c == ':' || c == '/')
                     .map(|i| &name[i + 1..])
-                    .unwrap_or(&name);
+                    .unwrap_or(name);
                 self.define_module(name.into(), module.expose(expose));
             }
             UseAs::Star => {
@@ -557,7 +556,7 @@ impl<'a> Scope {
         for (name, value) in &*self.functions.lock().unwrap() {
             let name = name.to_string();
             result.insert(
-                Value::Literal(name.clone(), Quotes::Double),
+                CssString::new(name.clone(), Quotes::Double).into(),
                 Value::Function(name, Some(value.clone())),
             );
         }
@@ -571,7 +570,7 @@ impl<'a> Scope {
         for (name, value) in &*self.variables.lock().unwrap() {
             if name != &Name::from_static("@scope_name@") {
                 result.insert(
-                    Value::Literal(name.to_string(), Quotes::Double),
+                    CssString::new(name.to_string(), Quotes::Double).into(),
                     value.clone(),
                 );
             }
@@ -917,25 +916,25 @@ pub mod test {
         s: &[(&'static str, &str)],
         expression: &[u8],
     ) -> Result<String, crate::Error> {
-        use super::{Scope, ScopeRef};
+        use super::ScopeRef;
         use crate::parser::value::value_expression;
         use crate::parser::{code_span, ParseError};
         use crate::sass::Name;
         use nom::bytes::complete::tag;
         use nom::sequence::terminated;
         let f = Default::default();
-        let scope = Scope::new_global(f);
+        let scope = ScopeRef::new_global(f);
         for &(name, val) in s {
             let val = value_expression(code_span(val.as_bytes()));
             scope.define(
                 Name::from_static(name),
-                &ParseError::check(val)?.evaluate(ScopeRef::new_global(f))?,
+                &ParseError::check(val)?.evaluate(scope.clone())?,
             );
         }
         let expr =
             terminated(value_expression, tag(";"))(code_span(expression));
         Ok(ParseError::check(expr)?
-            .evaluate(ScopeRef::dynamic(scope))?
+            .evaluate(scope)?
             .format(f)
             .to_string())
     }
